<?php

namespace Drupal\xtcguzzle\Plugin\XtcHandler;


use Drupal\Component\Serialization\Json;
use Drupal\xtc\PluginManager\XtcHandler\XtcHandlerPluginBase;
use Drupal\xtc\XtendedContent\API\XtcLoaderServer;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7\Uri;

/**
 * Plugin implementation of the xtc_handler for File.
 *
 */
abstract class GuzzleBase extends XtcHandlerPluginBase
{

  /**
   * @var string
   */
  protected $url;

  /**
   * @var Uri
   */
  protected $uri;

  /**
   * @var Client
   */
  protected $client;

  protected function initProcess(){
    $this->buildClient();
  }

  protected function runProcess(){
    $this->getStream();
  }

  protected function adaptContent(){
  }

  protected function buildClient(): GuzzleBase {
    $this->client = New Client($this->options);
    return $this;
  }

  protected function getStream() {
    $qs = $this->options['qs'] ?? '';
    $qs = \Drupal::request()->getQueryString() ?? $qs;
    $request = (!empty($qs)) ? $this->url . '?' . $qs : $this->url;
    try {
      $stream = $this->client->get($request);
      $this->content = $stream->getBody()->getContents();
    } catch (\Exception $exception) {
      $this->content = Json::encode(['Exception' => $exception->getMessage()]);
    }
  }

  public function values() {
    return Json::decode($this->content) ?? NULL;
  }

  /**
   * @param array $options
   *
   * @return \Drupal\xtc\PluginManager\XtcHandler\XtcHandlerPluginBase
   */
  public function setOptions($options = []): XtcHandlerPluginBase {
    parent::setOptions($options);
    $this->setUrl();
    $this->options['base_uri'] = $this->getUrl();
    $this->options['headers']['auth_token'] = $this->getToken();
    return $this;
  }

  /**
   * @return string
   */
  public function getUrl() {
    return $this->url;
  }

  public function setUrl(): GuzzleBase {
    $server = XtcLoaderServer::load($this->profile['server']);
    $env = $server['connection'][$server['env']];

    if ('self' == $env['server']) {
      $requestServer = \Drupal::request()->server;
      $protocole = ($requestServer->get('HTTPS')) ? 'https' : 'http';
      $domain = ($requestServer->get('HTTP_HOST')) ?? '';
      $port = ($requestServer->get('SERVER_PORT')) ? ':' . $requestServer->get('SERVER_PORT') : '';
    }
    else {
      $protocole = ($env['tls']) ? 'https' : 'http';
      $domain = $env['host'];
      $port = (!empty($env['port'])) ? ':' . $env['port'] : '';
    }
    $base_url = $protocole . '://' . $domain . $port;

    if (empty($this->method) && !empty($this->options['method'])) {
      $this->method = $this->options['method'];
    }
    if (empty($this->method) && !empty($this->profile['method'])) {
      $this->method = $this->profile['method'];
    }
    if (!empty($this->profile['query'])) {
      $this->method = $this->method . '/' . $this->profile['query'];
    }
    $this->endpoint = $env['endpoint'];
    $this->uri =
      (!empty($this->endpoint)) ? $this->endpoint . '/' . $this->method
        : $this->method;
    $this->url = $base_url . '/' . $this->uri;
    return $this;
  }

  /**
   * @return string
   */
  public function getToken(): string {
    //    return (isset($this->clientProfile['token'])) ? $this->clientProfile['token'] : '';
    return '';
  }

  /**
   * @return Uri
   */
  public function getUri() {
    return $this->uri;
  }

}
